--------------------------------------------------------------
-- ejecutar primero la sentencia create database warehouse
-- una vez logeado en la base de datos, ejecutar el resto del script
-- para generar datos, ejecutar la última línea comentada "--call public.test_data_generator()"
-----------------------------------------------------------
-- creación de la bbdd
-----------------------------------------------------------
--
CREATE DATABASE warehouse;
--
--
DROP SCHEMA IF EXISTS almacen CASCADE;
DROP SCHEMA IF EXISTS asociado CASCADE;
DROP SCHEMA IF EXISTS orden CASCADE;
DROP SCHEMA IF EXISTS factura CASCADE;
DROP SCHEMA IF EXISTS diccionario CASCADE;
CREATE SCHEMA IF NOT EXISTS asociado;
CREATE SCHEMA IF NOT EXISTS factura;
CREATE SCHEMA IF NOT EXISTS orden;
CREATE SCHEMA IF NOT EXISTS almacen;
CREATE SCHEMA IF NOT EXISTS diccionario;
-----------------------------------------------------------
-- definicion de las tablas                              --
-----------------------------------------------------------
-- TABLAS DE DICCIONARIO                                 --
CREATE TABLE diccionario.diccionario_tabla(
    tabla        VARCHAR(30),
    descripcion  VARCHAR(100),
    CONSTRAINT diccionario_pk PRIMARY KEY (tabla)
);
CREATE TABLE diccionario.diccionario_linea(
    tabla	VARCHAR(30),
    clave   VARCHAR(100), 
	valor	VARCHAR(100),
	CONSTRAINT diccionario_linea_pk PRIMARY KEY (tabla, clave),
	CONSTRAINT tabla_fk FOREIGN KEY (tabla) REFERENCES diccionario.diccionario_tabla(tabla)
);
CREATE TABLE diccionario.bloqueos_cliente (
	tabla			VARCHAR(30),
	CIF				VARCHAR(9) 	NOT NULL,
	fecha_inicio	DATE,
	fecha_fin		DATE,
	CONSTRAINT bloqueos_cliente_pk PRIMARY KEY (tabla, CIF),
	CONSTRAINT fecha_inicio_fin_chk CHECK (fecha_inicio > fecha_fin)
);
CREATE TABLE diccionario.pais(
	iso2		VARCHAR(2),
	iso3		VARCHAR(3),
	descripcion	VARCHAR(100) NOT NULL,
	CONSTRAINT pais_pk PRIMARY KEY (iso2)
);
CREATE TABLE diccionario.codigo_postal(
	iso2_pais	VARCHAR(2),
	cpostal		VARCHAR(10),
	descripcion VARCHAR(100) NOT NULL,
	CONSTRAINT codigo_postal_pk PRIMARY KEY (iso2_pais, cpostal),
	CONSTRAINT iso2_fk FOREIGN KEY (iso2_pais) REFERENCES diccionario.pais(iso2)
);
-----------------------------------------------------------
-- TABLAS DE ASOCIADO                                    --
-----------------------------------------------------------
CREATE TABLE IF NOT EXISTS asociado.cliente (
	cif						VARCHAR(9),
	nombre					VARCHAR(30),
	fecha_alta				DATE,
	fecha_baja				DATE,
	estado					VARCHAR(1),
	CONSTRAINT cliente_pk PRIMARY KEY(cif),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2', '3', '4'))
);
CREATE TABLE If NOT EXISTS asociado.direccion_cliente (
	cif						VARCHAR(9),
	tipo_direccion			VARCHAR(10),		
	calle					VARCHAR(30),
	numero					SMALLINT,
	complemento				VARCHAR(50),
	direccion_cpostal		VARCHAR(10) 	NOT NULL,
	pais					VARCHAR(02) 	NOT NULL,
	CONSTRAINT direccion_cliente_pk PRIMARY KEY(cif, tipo_direccion),
	CONSTRAINT cif_fk	FOREIGN KEY (cif) REFERENCES asociado.cliente(cif),
	CONSTRAINT direccion_cpostal_fk	FOREIGN KEY (pais, direccion_cpostal) REFERENCES diccionario.codigo_postal(iso2_pais, cpostal),
	CONSTRAINT tipo_direccion_chk CHECK (tipo_direccion = 'envio' or tipo_direccion = 'factura')
);
CREATE TABLE IF NOT EXISTS asociado.vendedor(
	cif						VARCHAR(9),
	nombre					VARCHAR(30),
	fecha_alta				DATE 	NOT NULL,
	fecha_baja				DATE,
	estado					BOOLEAN NOT NULL,
	CONSTRAINT vendedor_pk PRIMARY KEY(cif),
	CONSTRAINT estado_chk CHECK(estado in ('0', '1'))
);
CREATE TABLE If NOT EXISTS asociado.direccion_vendedor (
	cif						VARCHAR(9),
	tipo_direccion			VARCHAR(10),		
	calle					VARCHAR(30),
	numero					SMALLINT,
	complemento				VARCHAR(50),
	direccion_cpostal		VARCHAR(10)		NOT NULL,
	pais					VARCHAR(2) 		NOT NULL,
	CONSTRAINT direccion_vendedor_pk PRIMARY KEY(cif, tipo_direccion),
	CONSTRAINT cif_fk	FOREIGN KEY (cif) REFERENCES asociado.vendedor(cif),
	CONSTRAINT direccion_cpostal_fk	FOREIGN KEY (pais, direccion_cpostal) REFERENCES diccionario.codigo_postal(iso2_pais, cpostal),
	CONSTRAINT tipo_direccion_chk CHECK (tipo_direccion = 'factura')
);
-----------------------------------------------------------
-- TABLAS DE ALMACEN                                     --
-----------------------------------------------------------
CREATE TABLE IF NOT EXISTS almacen.pieza_maestro(
	id_pieza			VARCHAR(20),
	id_vendedor			VARCHAR(9),
	descripcion			VARCHAR(50),
	precio_neto_vend	NUMERIC(8,2) 	NOT NULL,
	precio_bruto_vend	NUMERIC(8,2) 	NOT NULL,
	estado				BOOLEAN 		NOT NULL, 
	CONSTRAINT pieza_maestro_pk 	PRIMARY KEY (id_pieza, id_vendedor),
	CONSTRAINT vend_pieza_fk		FOREIGN KEY (id_vendedor) REFERENCES asociado.vendedor (cif)
	);
CREATE TABLE IF NOT EXISTS almacen.pieza_local(
	id_pieza_gen			SERIAL,
	id_pieza				VARCHAR(20) 	NOT NULL,
	id_vendedor				VARCHAR(9)		NOT NULL,
	fecha_vigencia_ini		DATE			NOT NULL,
	fecha_vigencia_fin		DATE			NOT NULL,
	descripcion				VARCHAR(100),
	precio_neto_cliente		NUMERIC(8,2)	NOT NULL,	
	precio_bruto_cliente	NUMERIC(8,2)	NOT NULL,
	estado					BOOLEAN 		NOT NULL,
	stock					INTEGER			NOT NULL,
	CONSTRAINT pieza_local_pk 	PRIMARY KEY (id_pieza_gen),
	CONSTRAINT pieza_maest_loc_fk 	FOREIGN KEY (id_pieza, id_vendedor) REFERENCES  almacen.pieza_maestro (id_pieza, id_vendedor),
	CONSTRAINT precio_neto_cliente_chk CHECK (precio_neto_cliente >= 0),
	CONSTRAINT precio_bruto_cliente_chk CHECK (precio_neto_cliente >= 0),
	CONSTRAINT stock_chk CHECK (stock >= 0)
);
CREATE  INDEX pieza_local_ind ON almacen.pieza_local(id_pieza, id_vendedor );
CREATE TABLE IF NOT EXISTS almacen.localizacion(
	id_area			VARCHAR(4),
	id_seccion		VARCHAR(6),
	id_estanteria	VARCHAR(6),
	id_parte		INTEGER,
	estado			BOOLEAN,
	CONSTRAINT localizacion_pk 	PRIMARY KEY (id_area, id_seccion, id_estanteria),
	CONSTRAINT pieza_localiza_fk 	FOREIGN KEY(id_parte) REFERENCES almacen.pieza_local(id_pieza_gen)
);
CREATE TABLE IF NOT EXISTS orden.cli_backorder(
	backorder_clave		INTEGER,
	estado				VARCHAR(1) 	NOT NULL,
	fecha				DATE		NOT NULL,
	CONSTRAINT cli_backorder_pk 	PRIMARY KEY (backorder_clave)
);
CREATE TABLE IF NOT EXISTS orden.cli_backorder_estado(
	backorder_clave		INTEGER,
	estado				VARCHAR(1) 	NOT NULL,
	fecha_estado		DATE 		NOT NULL,
	CONSTRAINT cli_backorder_estado_pk PRIMARY KEY (backorder_clave,estado), 
	CONSTRAINT backorder_clave_fk FOREIGN KEY (backorder_clave) REFERENCES orden.cli_backorder(backorder_clave),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2'))
);
-----------------------------------------------------------
-- TABLAS DE ORDEN                                       --
-----------------------------------------------------------
CREATE TABLE IF NOT EXISTS orden.cliente_pedido(
	id_pedido		INTEGER,
	cif				VARCHAR(9),
	fecha			DATE,
	estado			VARCHAR(1),
	prioridad		BOOLEAN			NOT NULL, 
	CONSTRAINT cliente_pedido_pk 	PRIMARY KEY (id_pedido),
	CONSTRAINT cli_pedido_fk		FOREIGN KEY (cif) REFERENCES asociado.cliente (cif),
	CONSTRAINT id_pedido_chk CHECK (id_pedido > 0 AND id_pedido < 999999999)
);
CREATE TABLE IF NOT EXISTS orden.cliente_pedido_estado(
	id_pedido		INTEGER,
	estado			VARCHAR(1),
	fecha_estado1	DATE			NOT NULL,
	fecha_estado2	TIME			NOT NULL,
	CONSTRAINT cliente_pedido_estado_pk 	PRIMARY KEY (id_pedido, estado),
	CONSTRAINT id_pedido_fk		FOREIGN KEY (id_pedido) REFERENCES orden.cliente_pedido (id_pedido),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2', '3'))
);
CREATE TABLE IF NOT EXISTS orden.cliente_ped_li(
	id_pedido 			INTEGER,
	num_linea 			SMALLINT,
	id_pieza			INTEGER 		NOT NULL,
	cantidad			SMALLINT,
	coste_neto_unidad	NUMERIC(8,2) 	NOT NULL,
	coste_bruto_unidad	NUMERIC(8,2) 	NOT NULL,
	backorder			INTEGER,
	CONSTRAINT cliente_ped_li_pk PRIMARY KEY (id_pedido, num_linea),
	CONSTRAINT cli_ped_linea_fk FOREIGN KEY (id_pedido) REFERENCES  orden.cliente_pedido(id_pedido),
	CONSTRAINT cli_back_linea_fk FOREIGN KEY (backorder) REFERENCES  orden.cli_backorder(backorder_clave),
	CONSTRAINT cli_lin_pieza_fk FOREIGN KEY (id_pieza) REFERENCES   almacen.pieza_local(id_pieza_gen),
	CONSTRAINT num_linea_chk CHECK (num_linea > 0 AND num_linea < 999),
	CONSTRAINT cantidad CHECK (cantidad > 0 AND cantidad < 999)
);
-----------------------------------------------------------
-- TABLAS DE FACTURA                                     --
-----------------------------------------------------------
CREATE TABLE IF NOT EXISTS factura.cliente_factura(
	id_factura		INTEGER,
	id_cliente		VARCHAR(9) NOT NULL,
	fecha_factura	DATE NOT NULL,
	estado			VARCHAR(1) NOT NULL,
	CONSTRAINT cliente_factura_pk 	PRIMARY KEY (id_factura),
	CONSTRAINT id_factura_chk CHECK (id_factura > 0 AND id_factura < 999999999)
);
CREATE TABLE IF NOT EXISTS factura.cliente_factura_estado(
	id_factura		INTEGER,
	estado			VARCHAR(1),
	fecha_estado	DATE NOT NULL,
	CONSTRAINT cliente_factura_estado_pk 	PRIMARY KEY (id_factura, estado),
	CONSTRAINT id_factura_fk FOREIGN KEY (id_factura) REFERENCES factura.cliente_factura(id_factura),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2', '3'))
);
CREATE INDEX IF NOT EXISTS  c_ind_fech_fac_idx ON factura.cliente_factura (fecha_factura, id_cliente);
CREATE TABLE IF NOT EXISTS factura.c_factura_linea (
	id_factura	    INTEGER,
	num_linea		SMALLINT,
	fecha			DATE 			NOT NULL,
	estado			VARCHAR(1),
	total_neto		NUMERIC(8,2) 	NOT NULL,
	total_bruto		NUMERIC(8,2) 	NOT NULL,
	CONSTRAINT c_factura_linea_pk	PRIMARY KEY (id_factura, num_linea),
	CONSTRAINT cli_fac_linea_fk	FOREIGN KEY (id_factura) REFERENCES factura.cliente_factura(id_factura),
	CONSTRAINT num_linea_chk CHECK (num_linea > 0 AND num_linea < 999)
);
CREATE VIEW factura.c_ind_fech_fac_view AS 
 SELECT fact.id_factura, fact.id_cliente, fact.fecha_factura, estado.estado, estado.fecha_estado, 
         SUM(linea.total_neto) AS "total_neto",  SUM(linea.total_bruto)  AS "total_bruto"
 FROM factura.cliente_factura fact, factura.cliente_factura_estado estado, factura.c_factura_linea linea
 WHERE  fact.id_factura = estado.id_factura
   AND  fact.estado = estado.estado
   AND  fact.id_factura = linea.id_factura
 group by  1,2,3,4,5
 ORDER BY fecha_factura desc, id_cliente desc;
CREATE TABLE IF NOT EXISTS factura.c_factura_linea_estado (
	id_factura	    INTEGER,
	num_linea		SMALLINT,
	estado			VARCHAR(1),
	fecha_estado	DATE 		NOT NULL,
	CONSTRAINT c_factura_linea_estado_pk	PRIMARY KEY (id_factura, num_linea, estado),
	CONSTRAINT id_factura_num_linea_fk FOREIGN KEY (id_factura,num_linea) REFERENCES factura.c_factura_linea (id_factura, num_linea),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2'))
);
CREATE TABLE IF NOT EXISTS factura.c_factura_linea_pedido (
	id_factura	    INTEGER,
	num_linea		SMALLINT,
	id_pedido_orig  INTEGER,
	CONSTRAINT c_factura_linea_pedido_pk	PRIMARY KEY (id_factura, num_linea, id_pedido_orig),
	CONSTRAINT cli_fac_lin_ped_fk FOREIGN KEY(id_factura, num_linea) REFERENCES factura.c_factura_linea (id_factura, num_linea),
	CONSTRAINT cli_fac_ped_fk FOREIGN KEY(id_pedido_orig) REFERENCES orden.cliente_pedido (id_pedido)
);
CREATE TABLE IF NOT EXISTS factura.c_factura_linea_concepto (
	id_factura	    INTEGER,
	num_linea		SMALLINT,
	concepto		VARCHAR(30),
	CONSTRAINT c_factura_linea_concepto_pk	PRIMARY KEY (id_factura, num_linea, concepto),
	CONSTRAINT cli_fac_lin_con_fk FOREIGN KEY(id_factura, num_linea) REFERENCES factura.c_factura_linea (id_factura, num_linea)
);
CREATE FUNCTION factura.factura_linea_chk()  RETURNS trigger AS $func$
BEGIN
   PERFORM * 
   	from factura.c_factura_linea_pedido 
	WHERE id_factura = NEW.id_factura 
	  AND num_linea = NEW.num_linea;
   IF FOUND THEN 
   	RAISE EXCEPTION 'YA EXISTE LINEA DE FACTURA DE TIPO PEDIDO'; 
   END IF;
   
   PERFORM * 
   	FROM factura.c_factura_linea_concepto 
	WHERE id_factura = NEW.id_factura 
	  AND num_linea = NEW.num_linea;
   IF FOUND THEN 
    RAISE EXCEPTION 'YA EXISTE LINEA DE FACTURA DE TIPO CONCEPTO'; 
   END IF;
   RETURN NEW;
END;
$func$  LANGUAGE plpgsql;
CREATE TRIGGER factura_linea_pedido_chk BEFORE INSERT ON factura.c_factura_linea_pedido
FOR EACH ROW EXECUTE PROCEDURE  factura.factura_linea_chk();
CREATE TRIGGER factura_linea_concepto_chk BEFORE INSERT ON factura.c_factura_linea_concepto
FOR EACH ROW EXECUTE PROCEDURE  factura.factura_linea_chk();
CREATE TABLE IF NOT EXISTS orden.vendedor_pedido(
	id_pedido		INTEGER,
 	cif				VARCHAR(9),
 	fecha			DATE,
	estado			VARCHAR(1),
 	CONSTRAINT vendedor_pedido_pk 	PRIMARY KEY (id_pedido),
 	CONSTRAINT vend_pedido_fk		FOREIGN KEY (cif) REFERENCES asociado.vendedor (cif),
 	CONSTRAINT id_pedido_chk CHECK (id_pedido > 0 AND id_pedido < 999999999)
);
CREATE TABLE IF NOT EXISTS orden.vendedor_pedido_estado(
	id_pedido		INTEGER,
	estado			VARCHAR(1),
	fecha_estado1	DATE			NOT NULL,
	fecha_estado2	TIME			NOT NULL,
	CONSTRAINT vendedor_pedido_estado_pk 	PRIMARY KEY (id_pedido, estado),
	CONSTRAINT id_pedido_fk		FOREIGN KEY (id_pedido) REFERENCES orden.vendedor_pedido (id_pedido),
	CONSTRAINT estado_chk	CHECK (estado in ('0', '1', '2'))
);
CREATE TABLE IF NOT EXISTS orden.vendedor_ped_li(
	id_pedido		INTEGER,
	num_linea		SMALLINT,
	id_pieza		VARCHAR(20)	 NOT NULL,
	id_vendedor		VARCHAR(9)	 NOT NULL,
	cantidad		SMALLINT,
	total_neto		NUMERIC(8,2) NOT NULL,
	total_bruto		NUMERIC(8,2) NOT NULL,
	backorder	INTEGER,
	CONSTRAINT vendedor_ped_li_pk 	PRIMARY KEY (id_pedido, num_linea),
	CONSTRAINT vend_ped_linea_fk 	FOREIGN KEY (id_pedido) REFERENCES  orden.vendedor_pedido(id_pedido),
	CONSTRAINT vend_lin_pieza_fk 	FOREIGN KEY (id_pieza, id_vendedor) REFERENCES  almacen.pieza_maestro(id_pieza, id_vendedor),
	CONSTRAINT vend_back_linea_fk 	FOREIGN KEY (backorder) REFERENCES  orden.cli_backorder(backorder_clave),
	CONSTRAINT num_linea_chk CHECK (num_linea > 0 AND num_linea < 999),
	CONSTRAINT cantidad_chk CHECK (cantidad > 0 AND cantidad < 999)
);
CREATE TABLE IF NOT EXISTS factura.vendedor_factura(
	id_factura		INTEGER,
	id_vendedor		VARCHAR(9) 		NOT NULL,
	fecha_factura	DATE 			NOT NULL,
	estado			VARCHAR(1) 		NOT NULL,
	total_neto_conf	NUMERIC (8,2) 	NOT NULL,
	total_bruto_conf	NUMERIC (8,2) NOT NULL,
	CONSTRAINT vendedor_factura_pk	PRIMARY KEY (id_factura),
	CONSTRAINT id_vendedor_fk 	FOREIGN KEY (id_vendedor) REFERENCES asociado.vendedor(cif),
	CONSTRAINT id_factura_chk CHECK (id_factura > 0 AND id_factura < 999999999)
);
CREATE TABLE IF NOT EXISTS factura.vendedor_factura_estado(
	id_factura		INTEGER,
	estado			VARCHAR(1) 	NOT NULL,
	fecha_estado	DATE 		NOT NULL,
	CONSTRAINT vendedor_factura_estado_pk	PRIMARY KEY (id_factura, estado),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2', '3', '4'))
);
CREATE INDEX IF NOT EXISTS  v_ind_fech_fac_idx ON factura.vendedor_factura (fecha_factura, id_vendedor);
CREATE TABLE IF NOT EXISTS factura.v_factura_linea(
	id_factura		INTEGER,	
	num_linea		SMALLINT,
	fecha			DATE 			NOT NULL,
	estado			VARCHAR(1) 		NOT NULL,
	total_neto		NUMERIC(8,2) 	NOT NULL,
	total_bruto		NUMERIC(8,2) 	NOT NULL,
	CONSTRAINT v_factura_linea_pk	PRIMARY KEY (id_factura, num_linea),
	CONSTRAINT vend_fac_linea_fk	FOREIGN KEY (id_factura) REFERENCES factura.vendedor_factura(id_factura),
 	CONSTRAINT num_linea_chk CHECK (num_linea > 0 AND num_linea < 999)
);
CREATE VIEW factura.v_ind_fech_fac_view AS 
SELECT fact.id_factura,  fact.id_vendedor, fact.fecha_factura, fact.estado,
	   SUM(linea.total_neto) AS "total_neto",  SUM(linea.total_bruto)  AS "total_bruto",
	   fact.total_neto_conf, fact.total_bruto_conf
FROM factura.vendedor_factura fact, factura.vendedor_factura_estado est, factura.v_factura_linea linea
WHERE fact.id_factura = est.id_factura
  AND fact.estado = est.estado
  AND  fact.id_factura = linea.id_factura
group by  1,2,3,4,8
ORDER BY fecha_factura desc, id_vendedor desc;
CREATE TABLE IF NOT EXISTS factura.v_factura_linea_estado(
	id_factura		INTEGER,	
	num_linea		SMALLINT,
	estado			VARCHAR(1) 	NOT NULL,
	fecha_estado	DATE 		NOT NULL,
	CONSTRAINT v_factura_linea_estado_pk	PRIMARY KEY (id_factura, num_linea, estado),
	CONSTRAINT id_factura_num_linea_fk FOREIGN KEY (id_factura,num_linea) REFERENCES factura.V_factura_linea (id_factura, num_linea),
	CONSTRAINT estado_chk CHECK (estado in ('0', '1', '2'))
);

CREATE TABLE IF NOT EXISTS factura.v_factura_linea_pedido(
	id_factura		INTEGER,	
	num_linea		SMALLINT,
	id_pedido_orig	INTEGER,
	CONSTRAINT v_factura_linea_pedido_pk	PRIMARY KEY (id_factura, num_linea, id_pedido_orig),
	CONSTRAINT vend_fac_lin_fk	FOREIGN KEY (id_factura, num_linea) REFERENCES factura.v_factura_linea(id_factura, num_linea),
	CONSTRAINT vend_fac_lin_ped_fk FOREIGN KEY(id_pedido_orig) REFERENCES orden.vendedor_pedido (id_pedido)
);
CREATE TABLE IF NOT EXISTS factura.v_factura_linea_concepto(
	id_factura		INTEGER,	
	num_linea		SMALLINT,
	id_concepto		VARCHAR(30),
	CONSTRAINT v_factura_linea_concepto_pk	PRIMARY KEY (id_factura, num_linea, id_concepto),
	CONSTRAINT vend_fac_lin_fk	FOREIGN KEY (id_factura, num_linea) REFERENCES factura.v_factura_linea(id_factura, num_linea)
);
CREATE FUNCTION factura.v_factura_linea_chk()  RETURNS TRIGGER AS $func$
BEGIN
   PERFORM * 
   	from factura.v_factura_linea_pedido 
	WHERE id_factura = NEW.id_factura 
	  AND num_linea = NEW.num_linea;
   IF FOUND THEN 
   	RAISE EXCEPTION 'YA EXISTE LINEA DE FACTURA DE TIPO PEDIDO '; 
   END IF;
   
   PERFORM * 
   	FROM factura.V_factura_linea_concepto 
	WHERE id_factura = NEW.id_factura 
	  AND num_linea = NEW.num_linea;
   IF FOUND THEN 
    RAISE EXCEPTION 'YA EXISTE LINEA DE FACTURA DE TIPO CONCEPTO'; 
   END IF;
   RETURN NEW;
END;
$func$  LANGUAGE plpgsql;
CREATE TRIGGER v_factura_linea_pedido_chk BEFORE INSERT ON factura.v_factura_linea_pedido
FOR EACH ROW EXECUTE PROCEDURE  factura.v_factura_linea_chk();
CREATE TRIGGER v_factura_linea_concepto_chk BEFORE INSERT ON factura.v_factura_linea_concepto
FOR EACH ROW EXECUTE PROCEDURE  factura.v_factura_linea_chk();
------------------------------------------------------------
CREATE MATERIALIZED VIEW IF NOT EXISTS asociado.v_cliente_estadistica_mensual AS
SELECT cif, date_part('year', fecha) as "anyo", date_part('month', fecha) as "mes", sum (coste_bruto_unidad * cantidad) as "total_bruto" , max(fecha)
FROM ORDEn.cliente_pedido ped, ORDEn.cliente_ped_li lin
WHERE ped.id_pedido = lin.id_pedido
GROUP BY 1,2,3;
CREATE MATERIALIZED VIEW IF NOT EXISTS asociado.v_cliente_estadistica_anual AS
SELECT cif, date_part('year', fecha) as "anyo", COUNT(*) AS "pedido", SUM (coste_bruto_unidad * cantidad) AS "total_bruto"
FROM ORDEn.cliente_pedido PED, ORDEn.cliente_ped_li lin
WHERE ped.id_pedido = lin.id_pedido
GROUP BY 1,2;
------------------------------------------------------------
--3.	Para poder identificar las localizaciones libres para guardar las piezas nuevas que entran, o para reorganizar el almacén,
--muestra todas las localizaciones activas disponibles con la parte contenida y su cantidad en caso de no estar vacía
CREATE VIEW almacen.inventario AS
SELECT id_area, id_seccion, id_estanteria, id_pieza, descripcion, stock
FROM almacen.localizacion localizacion LEFT JOIN almacen.pieza_local pieza on localizacion.id_parte = pieza.id_pieza_gen
WHERE  localizacion.estado is true;
--2.	Selecciona aquellas líneas de pedido que forman parte de una backorder, que todavía no han generado pedido al proveedor, 
--y para las cuales sigue sin haber  stock (por lo que deberán generar pedido a proveedor) mostrando 
--mostrando el id del pedido original, el id de la backorder, el id de la  pieza y su descripción, cantidad solicitada,y  stock actual.
CREATE VIEW orden.backorden_pendiente AS
SELECT linea.id_pedido, linea.backorder, pieza.id_pieza, pieza.descripcion, linea.cantidad, pieza.stock
FROM orden.cliente_ped_li linea,  almacen.pieza_local pieza
WHERE linea.backorder IS NOT NULL
  AND NOT EXISTS (SELECT * FROM orden.vendedor_ped_li vend_linea WHERE linea.backorder = vend_linea.backorder)
  AND linea.id_pieza = pieza.id_pieza_gen
  AND linea.cantidad > pieza.stock;
--1.	Detalle de un pedido de cliente que muestre el identificador de pedido, el identificador de cliente, el estado, 
--el número de factura al que pertenece en caso de esta facturado, el total neto y bruto, y todas sus líneas incluyendo 
--información de la parte, descripción de la parte, cantidad solicitada, y coste neto y bruto de la línea. Esta vista 
--podría ser usada en una aplicación para recuperar la información completa de un pedido.
--SELECT *
CREATE VIEW orden.v_detalle_pedido AS
SELECT pedido.id_pedido, pedido.cif, pedido.estado, factura.id_factura, 
       linea.num_linea, pieza.id_pieza, pieza.descripcion, linea.cantidad, 
       linea.coste_neto_unidad * linea.cantidad AS "total neto", linea.coste_bruto_unidad * linea.cantidad AS "total bruto"
FROM orden.cliente_ped_li linea, almacen.pieza_local pieza, 
     orden.cliente_pedido pedido LEFT JOIN factura.c_factura_linea_pedido factura 
	                                    ON pedido.id_pedido = factura.id_pedido_orig
WHERE linea.id_pedido = pedido.id_pedido
  and linea.id_pieza = pieza.id_pieza_gen;
---------------------------------------------------------------------------------
--•	rango_backorder, •	rango_pedido_vendedor, •	Rango_pedido_cliente,. Rango_factura_CLIENTE, Rango_factura_VENDEDOR
 CREATE OR REPLACE FUNCTION diccionario.id_generator (rango_in VARCHAR(30)) RETURNS INTEGER AS $$
 DECLARE 
    valor_actual INTEGER DEFAULT 0;
    valor_nuevo  INTEGER DEFAULT 0;
 BEGIN
    SELECT valor INTO valor_actual 
	  FROM diccionario.diccionario_linea 
	  WHERE tabla = rango_in
	  FOR UPDATE;
    IF NOT FOUND THEN
    	RAISE EXCEPTION 'ERROR: NO EXISTE LA SECUENCIA % PARA GENERAR ID', rango;
 		valor_nuevo := 0;
    ELSE
     	valor_nuevo := valor_actual + 1;
 		UPDATE  diccionario.diccionario_linea 
		  SET valor = valor_nuevo
		WHERE tabla = rango_in;
    END IF;
    RETURN valor_nuevo;
 END; $$LANGUAGE plpgsql;
-------------------------------------------------------------------------------------------------------

CREATE INDEX IF NOT EXISTS  v_detalle_pedido_idx ON orden.cliente_pedido (cif);
--------------------------------------------------------------------------------------------------------
DROP ROLE  IF EXISTS warehouse_admin;
CREATE role warehouse_admin WITH 
	SUPERUSER 
	NOCREATEDB
	CREATEROLE
	LOGIN 
	CONNECTION LIMIT 1
	PASSWORD 'a123456';

DROP ROLE IF EXISTS warehouse_operacion;
CREATE role warehouse_operacion WITH 
	PASSWORD 'a123456' VALID UNTIL '2023-12-31';
GRANT CONNECT ON DATABASE warehouse TO warehouse_operacion;
GRANT ALL PRIVILEGES ON SCHEMA almacen TO warehouse_operacion;
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA almacen TO warehouse_operacion;
GRANT ALL PRIVILEGES ON SCHEMA asociado TO warehouse_operacion;
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA asociado TO warehouse_operacion;
GRANT ALL PRIVILEGES ON SCHEMA factura TO warehouse_operacion;
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA factura TO warehouse_operacion;
GRANT ALL PRIVILEGES ON SCHEMA orden TO warehouse_operacion;
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA orden TO warehouse_operacion;

DROP ROLE IF EXISTS warehouse_lectura;
CREATE role warehouse_lectura WITH 
	LOGIN 
	PASSWORD 'a123456' VALID UNTIL '2023-12-31';	
GRANT CONNECT ON DATABASE warehouse TO warehouse_lectura;
GRANT USAGE  ON SCHEMA almacen TO warehouse_lectura;
GRANT SELECT ON TABLE almacen.inventario TO warehouse_lectura;
GRANT USAGE  ON SCHEMA asociado TO warehouse_lectura;
GRANT SELECT ON TABLE asociado.v_cliente_estadistica_mensual  TO warehouse_lectura;
GRANT SELECT ON TABLE asociado.v_cliente_estadistica_anuaL  TO warehouse_lectura;
GRANT USAGE  ON SCHEMA factura TO warehouse_lectura;
GRANT SELECT ON TABLE factura.c_ind_fech_fac_view  TO warehouse_lectura;
GRANT SELECT ON TABLE factura.v_ind_fech_fac_view  TO warehouse_lectura;
GRANT USAGE  ON SCHEMA orden TO warehouse_lectura;
GRANT SELECT ON TABLE orden.backorden_pendiente  TO warehouse_lectura;
GRANT SELECT ON TABLE orden.v_detalle_pedido   TO warehouse_lectura;

------------------------------------------------------------------------------------------------------------------------
-- Función  que genera datos de test. descomentar la línea   call public.test_data_generator() ; para ejecutar
------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE public.test_data_generator() AS $$
DECLARE
	in_pais diccionario.pais%ROWTYPE;
	in_cp   diccionario.codigo_postal%ROWTYPE;
	in_cliente asociado.cliente%ROWTYPE;
	in_direccion asociado.direccion_cliente%ROWTYPE;
	in_vendedor asociado.vendedor%ROWTYPE;
	in_vdireccion asociado.direccion_vendedor%ROWTYPE;
	in_maestro almacen.pieza_maestro%ROWTYPE;
	in_local almacen.pieza_local%ROWTYPE;
	in_localizacion almacen.localizacion%ROWTYPE;
	in_cliente_pedido  orden.cliente_pedido%ROWTYPE;
	in_cliente_pedido_estado orden.cliente_pedido_estado%ROWTYPE;
	in_cliente_ped_li orden.cliente_ped_li%ROWTYPE;
	in_cliente_factura factura.cliente_factura%ROWTYPE;
	in_cliente_factura_estado factura.cliente_factura_estado%ROWTYPE;
	in_c_factura_linea factura.c_factura_linea%ROWTYPE;
	in_c_factura_linea_pedido factura.c_factura_linea_pedido%ROWTYPE;
	in_c_factura_linea_concepto factura.c_factura_linea_concepto%ROWTYPE;
	contador INTEGER DEFAULT 0;
	contador2 INTEGER DEFAULT 0;
	contador_pedido INTEGER DEFAULT 0;
	part_serial_id INTEGER DEFAULT 0;
	cont_area INTEGER DEFAULT 0;
	cont_seccion  INTEGER DEFAULT 0;
	cont_estanteria  INTEGER DEFAULT 0;
BEGIN
RAISE NOTICE 'BEGIN';
--datos de diccionario
	in_pais.iso2 := 'ES';
	in_pais.iso3 := 'SPA';
	in_pais.descripcion := 'españa';
	INSERT INTO diccionario.pais VALUES (in_pais.*);
	in_cp.iso2_pais := 'ES';
	in_cp.cpostal := '08402';
	in_cp.descripcion := 'GRANOLLERS';
	INSERT INTO diccionario.codigo_postal VALUES (in_cp.*);
	in_cp.iso2_pais := 'ES';
	in_cp.cpostal := '09032';
	in_cp.descripcion := 'BARCELONA';
	INSERT INTO diccionario.codigo_postal VALUES (in_cp.*);

-- PROVEEDORES DE PIEZAS NO HAY TANTos como clientes por lo que repetimos pero reducimos a varios cientos, 500 activos, 100 cancelados
	contador :=0;
	in_vendedor.fecha_alta := current_date;
	in_vendedor.fecha_baja := NULL;
	in_vendedor.estado  := '1';		
	RAISE NOTICE 'Generando datos relativos a piezas...';
	WHILE contador < 501 LOOP
		contador := contador+1;
		in_vendedor.cif := 'B000' || TO_CHAR(contador,'FM09999');
		in_vendedor.nombre := 'nombre_cli' || TO_CHAR(contador,'9');
		INSERT INTO asociado.vendedor VALUES (in_vendedor.*);
		in_vdireccion.cif := in_vendedor.cif;
		in_vdireccion.tipo_direccion := 'factura';
		in_vdireccion.calle := ' calle factura';
		in_vdireccion.numero := contador;
		in_vdireccion.complemento := null;
		in_vdireccion.direccion_cpostal := '08402';
		in_vdireccion.pais	 := 'ES';
		INSERT INTO asociado.direccion_vendedor VALUES (in_vdireccion.*);		
		
--para cada vendedor activo, generamos 1000 piezas, cada una con 2 versiones		
			contador2 :=0;
			in_maestro.id_vendedor 	:= in_vendedor.cif;
			in_maestro.precio_neto_vend	:= 100.00;
			in_maestro.precio_bruto_vend := 121.00;
			in_maestro.estado := TRUE;	
			WHILE contador2 < 1000 LOOP
				contador2 := contador2+1;
				in_maestro.id_pieza := 'P00000000000000' || TO_CHAR(contador2,'FM09999');
				in_maestro.descripcion := 'PIEZA NUM'|| TO_CHAR(contador2,'09999');
				INSERT INTO almacen.pieza_maestro VALUES (in_maestro.*);		
				in_local.id_pieza := in_maestro.id_pieza;
				in_local.id_vendedor := in_maestro.id_vendedor;
				in_local.fecha_vigencia_ini := current_date - 6;
				in_local.fecha_vigencia_fin	:= current_date - 1;
				in_local.descripcion := in_maestro.descripcion || 'v1';
				in_local.precio_neto_cliente := 200.00;
				in_local.precio_bruto_cliente := 210.00;
				in_local.estado := true;
				in_local.stock := 0;
				INSERT INTO almacen.pieza_local VALUES (default, in_local.id_pieza, in_local.id_vendedor, in_local.fecha_vigencia_ini, in_local.fecha_vigencia_fin, in_local.descripcion,	in_local.precio_neto_cliente,
				in_local.precio_bruto_cliente, in_local.estado,	in_local.stock);
				in_local.fecha_vigencia_ini := current_date ;
				in_local.fecha_vigencia_fin	:= current_date + 6;
				in_local.descripcion := in_maestro.descripcion || 'v2';
				INSERT INTO almacen.pieza_local VALUES (default, in_local.id_pieza, in_local.id_vendedor, in_local.fecha_vigencia_ini, in_local.fecha_vigencia_fin, in_local.descripcion,	in_local.precio_neto_cliente,
				in_local.precio_bruto_cliente, in_local.estado,	in_local.stock) RETURNING id_pieza_gen INTO part_serial_id;
--				raise notice 'id_pieza_gen:%', part_serial_id;
-- ademas a cada pieza activa asignamos uina localizacion, y cada vez que hagamos cambio de localizacion generamos una vacía

					cont_estanteria = cont_estanteria+1;
					IF cont_estanteria = 999 THEN
					   cont_estanteria := 0;
					   cont_seccion := cont_seccion+1;
					END IF;
					IF cont_seccion = 999 THEN
						cont_seccion := 0; 
						cont_area = cont_area+1;
					END IF;
					in_localizacion.id_area	:=	'A' || TO_CHAR(cont_area, 'FM099');
					in_localizacion.id_seccion	:=	'S' || TO_CHAR(cont_seccion,'FM099');
					in_localizacion.id_estanteria	:=	'E' || TO_CHAR(cont_estanteria,'FM099');
 					--RAISE NOTICE ' id_area:% id_seccion:% id_estanteria:%', in_localizacion.id_area, in_localizacion.id_seccion, in_localizacion.id_estanteria
-- 					RAISE NOTICE ' EL SUBSTRING:% ', SUBSTRING(in_maestro.id_pieza,15,5);
					in_localizacion.id_parte :=	part_serial_id;
					in_localizacion.estado :=	TRUE;
					INSERT INTO almacen.localizacion VALUES (in_localizacion.*);
					
 			END LOOP;
	
	END LOOP;
	RAISE NOTICE 'Generando datos adicionales de proveedor...';
	in_vendedor.estado  := '0';	
	in_vendedor.fecha_alta := current_date-2;
	in_vendedor.fecha_baja := current_date-1;
	WHILE contador < 601 LOOP
		contador := contador+1;
		in_vendedor.cif := 'B000' || TO_CHAR(contador,'FM09999');
		in_vendedor.nombre := 'nombre_cli' || TO_CHAR(contador,'09999');
		INSERT INTO asociado.vendedor VALUES (in_vendedor.*);				
		in_vdireccion.cif := in_vendedor.cif;
		in_vdireccion.tipo_direccion := 'factura';
		in_vdireccion.calle := ' calle factura';
		in_vdireccion.numero := contador;
		in_vdireccion.complemento := null;
		in_vdireccion.direccion_cpostal := '08402';
		in_vdireccion.pais	 := 'ES';
		INSERT INTO asociado.direccion_vendedor VALUES (in_vdireccion.*);
	END LOOP;
-- generaremos 50000 piezas, de las cuales 20.000 tendran una sola version, 20.000 tendras 2 version y 10.000 no tendran pieza local y estarn bloqueadas
--========================= CLIENTE  ==========================---
-- genera clientes 8000 clientes activos, 1000 clientes bloqueados de ordenes y 1000 canceldos, y direcciones
	contador :=0;
	contador2 :=0;
	contador_pedido :=0;
	in_cliente.fecha_alta := current_date;
	in_cliente.fecha_baja := NULL;
	in_cliente.estado  := '1';		
	RAISE NOTICE 'Generando clientes con pedidos y facturas...';
	WHILE contador < 8001 LOOP
		contador := contador+1;
		in_cliente.cif := 'A000' || TO_CHAR(contador,'FM09999');
		in_cliente.nombre := 'nombre_cli' || TO_CHAR(contador,'9');
		INSERT INTO asociado.cliente VALUES (in_cliente.*);
		in_direccion.cif := in_cliente.cif;
		in_direccion.tipo_direccion := 'envio';
		in_direccion.calle := ' calle envio';
		in_direccion.numero := contador;
		in_direccion.complemento := null;
		in_direccion.direccion_cpostal := '08402';
		in_direccion.pais	 := 'ES';
		INSERT INTO asociado.direccion_cliente VALUES (in_direccion.*);
		in_direccion.tipo_direccion := 'factura';
		in_direccion.calle := ' calle factura';
		in_direccion.numero := contador;
		INSERT INTO asociado.direccion_cliente VALUES (in_direccion.*);
		
-- PARA CADA CLIENTE INSERTAMOS 40 pedidos activos, PARA CADA pedido 2 PARTES, y 10 no activas sin partes
-- las order activas estan en una factura para ese cliente => primero fact, luego orders, luego lineas de fac con las orders
			in_cliente_factura.id_factura:= contador;		
			in_cliente_factura.id_cliente	:= in_cliente.cif;
			in_cliente_factura.fecha_factura := current_date;
			in_cliente_factura.estado := '1';
--			in_cliente_factura.total_neto := null;
--			in_cliente_factura.total_bruto := null;
			INSERT INTO factura.cliente_factura VALUES (in_cliente_factura.*);
			in_cliente_factura_estado.id_factura	:= 	in_cliente_factura.id_factura;
			in_cliente_factura_estado.estado:= '1';
			in_cliente_factura_estado.fecha_estado	:= CURRENT_DATE;
			INSERT INTO factura.cliente_factura_estado VALUES (in_cliente_factura_estado.*);

			contador2 := 0;
			WHILE contador2 < 41 LOOP
				contador2 := contador2+1;
				contador_pedido := contador_pedido+1;
				in_cliente_pedido.id_pedido := contador_pedido;
				in_cliente_pedido.cif		:= in_cliente.cif;
				in_cliente_pedido.fecha	 	:= current_date;
				in_cliente_pedido.estado	:= '1';
				in_cliente_pedido.prioridad	:= false;
--				in_cliente_pedido.total_neto := 400.00;
--				in_cliente_pedido.total_bruto := 420.00;
				INSERT INTO  orden.cliente_pedido VALUES (in_cliente_pedido.*);
				
				in_c_factura_linea.id_factura	:=   in_cliente_factura.id_factura;
				in_c_factura_linea.num_linea := 	contador2;
				in_c_factura_linea.fecha	:= current_date;
				in_c_factura_linea.estado	:= '1';
				in_c_factura_linea.total_neto	:= 400.00;
				in_c_factura_linea.total_bruto	:= 420.00;
				INSERT INTO  factura.c_factura_linea VALUES (in_c_factura_linea.*);
				IF contador2 = 40 THEN
					in_c_factura_linea_concepto.id_factura := in_c_factura_linea.id_factura;
					in_c_factura_linea_concepto.num_linea := in_c_factura_linea.num_linea;
					in_c_factura_linea_concepto.concepto := 'transporte';
					INSERT INTO  factura.c_factura_linea_concepto VALUES (in_c_factura_linea_concepto.*);
				ELSE 		
					in_c_factura_linea_pedido.id_factura := in_c_factura_linea.id_factura;
					in_c_factura_linea_pedido.num_linea := in_c_factura_linea.num_linea;
					in_c_factura_linea_pedido.id_pedido_orig := in_cliente_pedido.id_pedido;
					INSERT INTO  factura.c_factura_linea_pedido VALUES (in_c_factura_linea_pedido.*);
				END IF;
				
				in_cliente_pedido_estado.id_pedido := in_cliente_pedido.id_pedido;
				in_cliente_pedido_estado.estado := '1';
				in_cliente_pedido_estado.fecha_estado1	:= CURRENT_DATE;
				in_cliente_pedido_estado.fecha_estado2	:= CURRENT_TIME;				
				INSERT INTO  orden.cliente_pedido_estado VALUES (in_cliente_pedido_estado.*);		
				in_cliente_ped_li.id_pedido := in_cliente_pedido.id_pedido;
				in_cliente_ped_li.num_linea := 1;
				in_cliente_ped_li.id_pieza	:= part_serial_id;
				in_cliente_ped_li.cantidad	:= 1;
				in_cliente_ped_li.coste_neto_unidad := 200.00;
				in_cliente_ped_li.coste_bruto_unidad  := 210.00;
				in_cliente_ped_li.backorder	:= NULL;				
				INSERT INTO  orden.cliente_ped_li VALUES (in_cliente_ped_li.*);
				in_cliente_ped_li.id_pedido := in_cliente_pedido.id_pedido;
				in_cliente_ped_li.num_linea := 2;
				in_cliente_ped_li.id_pieza	:= part_serial_id-10000;
				in_cliente_ped_li.cantidad	:= 2;
				in_cliente_ped_li.coste_neto_unidad := 200.00;
				in_cliente_ped_li.coste_bruto_unidad  := 210.00;
				in_cliente_ped_li.backorder	:= NULL;				
				INSERT INTO  orden.cliente_ped_li VALUES (in_cliente_ped_li.*);			
			END LOOP;
			WHILE contador2 < 51 LOOP
				contador2 := contador2+1;
				contador_pedido := contador_pedido+1;
				in_cliente_pedido.id_pedido := contador_pedido;
				in_cliente_pedido.cif		:= in_cliente.cif;
				in_cliente_pedido.fecha	 	:= current_date;
				in_cliente_pedido.estado	:= '0';
				in_cliente_pedido.prioridad	:= false;
--				in_cliente_pedido.total_neto := 0;
--				in_cliente_pedido.total_bruto := 0;
				INSERT INTO  orden.cliente_pedido VALUES (in_cliente_pedido.*);
				
				
				
				in_cliente_pedido_estado.id_pedido := in_cliente_pedido.id_pedido;
				in_cliente_pedido_estado.estado := '0';
				in_cliente_pedido_estado.fecha_estado1	:= CURRENT_DATE;
				in_cliente_pedido_estado.fecha_estado2	:= CURRENT_TIME;				
				INSERT INTO  orden.cliente_pedido_estado VALUES (in_cliente_pedido_estado.*);		
				in_cliente_ped_li.id_pedido := in_cliente_pedido.id_pedido;
				in_cliente_ped_li.num_linea := 1;
				in_cliente_ped_li.id_pieza	:= part_serial_id;
				in_cliente_ped_li.cantidad	:= 1;
				in_cliente_ped_li.coste_neto_unidad := 200.00;
				in_cliente_ped_li.coste_bruto_unidad  := 210.00;
				in_cliente_ped_li.backorder	:= NULL;			
				INSERT INTO  orden.cliente_ped_li VALUES (in_cliente_ped_li.*);
				
			END LOOP;		

	END LOOP;
	RAISE NOTICE 'Generando clientes adicionales ...';
	in_cliente.estado  := '2';	
	in_cliente.fecha_alta := current_date-1;
	in_cliente.fecha_baja := NULL;
	WHILE contador < 9001 LOOP
		contador := contador+1;
		in_cliente.cif := 'A000' || TO_CHAR(contador,'FM09999');
		in_cliente.nombre := 'nombre_cli' || TO_CHAR(contador,'09999');
		INSERT INTO asociado.cliente VALUES (in_cliente.*);				
		in_direccion.cif := in_cliente.cif;
		in_direccion.tipo_direccion := 'envio';
		in_direccion.calle := ' calle envio';
		in_direccion.numero := contador;
		in_direccion.complemento := null;
		in_direccion.direccion_cpostal := '08402';
		in_direccion.pais	 := 'ES';
		INSERT INTO asociado.direccion_cliente VALUES (in_direccion.*);
		in_direccion.tipo_direccion := 'factura';
		in_direccion.calle := ' calle factura';
		in_direccion.numero := contador;
		INSERT INTO asociado.direccion_cliente VALUES (in_direccion.*);
	END LOOP;
	in_cliente.fecha_alta := current_date-2;
	in_cliente.fecha_baja := current_date-1;
	in_cliente.estado  := '0';	
	RAISE NOTICE 'Generando direcciones de cliente';
	WHILE contador < 10001 LOOP
		contador := contador+1;
		in_cliente.cif := 'A000' || TO_CHAR(contador,'FM09999');
		in_cliente.nombre := 'nombre_cli' || TO_CHAR(contador,'09999');
		INSERT INTO asociado.cliente VALUES (in_cliente.*);
		in_direccion.cif := in_cliente.cif;
		in_direccion.tipo_direccion := 'envio';
		in_direccion.calle := ' calle envio';
		in_direccion.numero := contador;
		in_direccion.complemento := null;
		in_direccion.direccion_cpostal := '09032';
		in_direccion.pais	 := 'ES';
		INSERT INTO asociado.direccion_cliente VALUES (in_direccion.*);
		in_direccion.tipo_direccion := 'factura';
		in_direccion.calle := ' calle factura';
		in_direccion.numero := contador;
		INSERT INTO asociado.direccion_cliente VALUES (in_direccion.*);
	END LOOP;


RAISE NOTICE 'Generacion de datos de test completada';

END; $$LANGUAGE  plpgsql;
--call public.test_data_generator() ;





